class QuickVote < Election
  before_validation :build_candidate_names
  validates_uniqueness_of :name

  attr_accessor :candidate_names
  attr_accessor :reviewed
  
  def initialize(params={})
    super
    self.startdate = Time.now
    self.active = 1
    self.anonymous = 1 unless self.anonymous
  end
  
  def validate
    if @candidate_names.length < 2
      errors.add(nil, "You must list at least two candidates.")
    end

    @candidate_names.each do |c|
      unless c.instance_of? String
        errors.add(nil, "Candidates must be strings")
        next
      end
      c.strip!
      if c.length == 0
        errors.add(nil, "Candidate name must not be empty")
        next
      end
    end if @candidate_names
 
    if @candidate_names and @candidate_names.uniq!
      errors.add(nil, "Candidates must all be unique")
    end

    if name =~ /[^A-Za-z0-9]/
      errors.add(:name, "must only include numbers and letters.")
    end
    if name =~ /^[0-9]+$/
      errors.add(:name, "must not be a number")
    end
    
    if name =~ /^(create|index|confirm|change|results)$/
      errors.add(:name, " is a reserved word.")
    end

    if enddate < startdate
      errors.add(nil, "QuickVotes can't end before they start")
    end
  end

  def name
    read_attribute( :name ).downcase() if read_attribute( :name )
  end

  def reviewed?
    reviewed.to_i == 1
  end

  def build_candidate_names
    @candidate_names ||= []
    if @candidate_names.empty? and not self.candidates.empty?
        @candidate_names = self.candidates.collect {|c| c.name}
    end
  end

  def create_candidates
    return unless errors.empty?
    
    # delete the candidates
    candidates.each {|c| c.destroy}

    # create the new list based on the names
    @candidate_names.each do |name|
      candidate = Candidate.new({:name => name})
      self.candidates << candidate
    end
  end

  ### Convert a shortname or id into a QuickVote
  def self.ident_to_quickvote(ident)
    return nil unless ident
    if ident.match(/^\d+$/)
      quickvote = QuickVote.find(ident)
    else
      quickvote = QuickVote.find(:all, :conditions => ["name = ?", ident])[0]
    end

    return quickvote
  end
end
